//
//  AppDelegate.swift
//  Example
//
//  Created by Connor Ricks on 2022-01-05.
//

import UIKit
import Combine
import RealmSwift

class Foo: Object {
    @Persisted(primaryKey: true) var id: UUID
    @Persisted(originProperty: "foo") var bars: LinkingObjects<Bar>
}

class Bar: Object {
    @Persisted(primaryKey: true) var id: UUID
    @Persisted var foo: Foo?
}


@main
class AppDelegate: UIResponder, UIApplicationDelegate {

    var cancellable: AnyCancellable?

    func application(_ application: UIApplication, didFinishLaunchingWithOptions launchOptions: [UIApplication.LaunchOptionsKey: Any]?) -> Bool {
        let realm = try! Realm()

        let foo = Foo()
        try! realm.write {
            realm.add(foo)
        }

        cancellable = valuePublisher(foo).sink { completion in
            print("End of publisher.")
        } receiveValue: { foo in
            print("Foo Changed: \(foo.bars.count)") // Expected 3 hits... (Never hit!)
        }

        try! realm.write {
            let barOne = Bar()
            barOne.foo = foo
            realm.add(barOne)

            let barTwo = Bar()
            barTwo.foo = foo
            realm.add(barTwo)

            let barThree = Bar()
            barThree.foo = foo
            realm.add(barThree)
        }

        print("End Write:", foo.bars.count) // Expected Output - "End Write: 3"

        return true
    }

    func application(_ application: UIApplication, configurationForConnecting connectingSceneSession: UISceneSession, options: UIScene.ConnectionOptions) -> UISceneConfiguration {
        return UISceneConfiguration(name: "Default Configuration", sessionRole: connectingSceneSession.role)
    }
}

